--
--   Unit Switcher v2.0
--   Change the ingame speed & fuel units between Metric, Imperial and US
--
-- @original Knuuud and Daveaich (ls-uk.info)
-- @author   Decker    (ls-uk.info, Decker_MMIV)
-- @date     2011-09-05
--
-- @comment Found problems or bugs? Please post them in the www.ls-uk.info support-thread for this mod. Thank you.
--          This mod is placed in the section 'Other', which at time of writing is; http://www.ls-uk.info/downloads/6
--
-- @history         - Knuuud suggested I took over the UnitSwitcher
--          v2.00   - Cleaned up some code.
--                  - Gave each unit (speed, fuel, currency) their individual ALT+<key>, where the <key> can be configured.
--                  - Added load/save functionality, which is not bound to a single vehicle.
--          v2.01   - Misc. tweaks in the load/save functions.
--

Units = {}
--
Units.speedType     = -1;
Units.fuelType      = -1;
Units.currencyType  = -1;
--
Units.speedFactor = 1;
Units.fuelFactor  = 1;
-- Use original texts at startup.
Units.speedUnit = g_i18n:getText("speedometer");
Units.fuelUnit  = g_i18n:getText("fluid_unit_long");

function Units:loadMap(name)
    if self.initialized then
        -- Only initialize one time.
        return;
    end;

    -- First unit-name will be the original, given by the game-engine.
    Units.currencies    = { g_i18n:getText("Currency_symbol") };
    Units.pricesperton  = { g_i18n:getText("PricePerTon") };
    -- The additional unit-names are read from the modDesc.XML's <l10n> texts.
    for _,v in pairs(Utils.splitString(" ", Utils.getNoNil(g_i18n:getText("currencies"),""))) do
        table.insert(Units.currencies, v);
    end;
    for _,v in pairs(Utils.splitString(" ", Utils.getNoNil(g_i18n:getText("pricesperton"),""))) do
        table.insert(Units.pricesperton, v);
    end;
    if (table.getn(Units.currencies) ~= table.getn(Units.pricesperton)) then
        print("WARNING - Units, 'currencies' and 'pricesperton' do not contain the same number of elements!");
    end;
    --
    self.msgTimer = 0;
    self:load();
    --
    self.initialized = true;
end;

function Units:deleteMap()
end;

function Units:load(xmlFile)
end;

function Units:delete()
end;

function Units:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Units:keyEvent(unicode, sym, modifier, isDown)
end;

function Units:update(dt)
    if (self.msgTimer > 0) then
        self.msgTimer = self.msgTimer - dt;
    end;

    -- Only accept input-binding-keys when ALT-key is also pressed.
    if Input.isKeyPressed(Input.KEY_lalt) then
        if InputBinding.hasEvent(InputBinding.UnitsSpeed) then
            Units.speedType = (Units.speedType + 1) % 2;
            self:setSpeedUnit(Units.speedType);
            Units:setInfo(string.format(g_i18n:getText("speedChanged"), Units.speedUnit));
        elseif InputBinding.hasEvent(InputBinding.UnitsFuel) then
            Units.fuelType = (Units.fuelType + 1) % 3;
            self:setFuelUnit(Units.fuelType);
            Units:setInfo(string.format(g_i18n:getText("fuelChanged"), Units.fuelUnit));
        elseif InputBinding.hasEvent(InputBinding.UnitsCurrency) then
            Units.currencyType = (Units.currencyType + 1) % math.min(table.getn(Units.currencies), table.getn(Units.pricesperton));
            self:setCurrencyUnit(Units.currencyType);
            Units:setInfo(string.format(g_i18n:getText("currencyChanged"), g_i18n:getText("Currency_symbol")));
        end;
    end;
end;

function Units:setSpeedUnit(tpe)
    if (tpe == 0) then
        Units.speedFactor = 1;
        Units.speedUnit = g_i18n:getText("speed_kmh");
    elseif (tpe == 1) then
        Units.speedFactor = 1.6;
        Units.speedUnit = g_i18n:getText("speed_mph");
    end;
end;

function Units:setFuelUnit(tpe)
    if (tpe == 0) then
        Units.fuelFactor = 1;
        Units.fuelUnit  = g_i18n:getText("fuel_metric");
    elseif (tpe == 1) then
        Units.fuelFactor = 4.55;
        Units.fuelUnit  = g_i18n:getText("fuel_imperial");
    elseif (tpe == 2) then
        Units.fuelFactor = 3.79;
        Units.fuelUnit  = g_i18n:getText("fuel_imperial_us");
    end;
end;

function Units:setCurrencyUnit(tpe)
    if (tpe >= 0 and tpe < math.min(table.getn(Units.currencies), table.getn(Units.pricesperton))) then
        g_i18n.globalI18N.texts["Currency_symbol"]  = Units.currencies[tpe+1];
        g_i18n.globalI18N.texts["PricePerTon"]      = Units.pricesperton[tpe+1];
    end;
end;

function Units:setInfo(msg)
    self.msgTxt = msg;
    self.msgTimer = 3000;
end;

function Units:draw()
    -- Only add help-buttons when left-ALT is pressed, and helpbox is visible.
    if Input.isKeyPressed(Input.KEY_lalt) and g_currentMission.showHelpText then
        g_currentMission:addHelpButtonText(g_i18n:getText("UnitsSpeed"),    InputBinding.UnitsSpeed);
        g_currentMission:addHelpButtonText(g_i18n:getText("UnitsCurrency"), InputBinding.UnitsCurrency);
        g_currentMission:addHelpButtonText(g_i18n:getText("UnitsFuel"),     InputBinding.UnitsFuel);
    end;
    --
    if (self.msgTimer > 0) then
        setTextAlignment(RenderText.ALIGN_CENTER);
        setTextBold(false);
        setTextColor(1,1,1, 1);
        renderText(0.50, 0.07, 0.03, self.msgTxt);
        setTextAlignment(RenderText.ALIGN_LEFT);
    end;
end;

function Units:load()
    -- 'self' is in context of Units object
    local path = getUserProfileAppPath() .. "savegame" .. g_careerScreen.selectedIndex;
    --print("Units:load() "..path);
    local rootTag = "UnitSwitcher";
    local xmlFile = loadXMLFile(nil, path.."/UnitSwitcher.xml");
    Units.speedType     = Utils.getNoNil(getXMLInt(xmlFile, rootTag..".config#speedType"),    Units.speedType);
    Units.fuelType      = Utils.getNoNil(getXMLInt(xmlFile, rootTag..".config#fuelType"),     Units.fuelType);
    Units.currencyType  = Utils.getNoNil(getXMLInt(xmlFile, rootTag..".config#currencyType"), Units.currencyType);
    delete(xmlFile);
    --
    self:setSpeedUnit(Units.speedType);
    self:setFuelUnit(Units.fuelType);
    self:setCurrencyUnit(Units.currencyType);
end;

function Units:save()
    -- 'self' is in context of the g_careerScreen object
    local path = self.savegames[self.selectedIndex].savegameDirectory;
    --print("Units:save() "..path);
    local rootTag = "UnitSwitcher";
    local xmlFile = createXMLFile(nil, path.."/UnitSwitcher.xml", rootTag);
    if (Units.speedType    > -1) then setXMLInt(xmlFile, rootTag..".config#speedType",     Units.speedType);    end;
    if (Units.fuelType     > -1) then setXMLInt(xmlFile, rootTag..".config#fuelType",      Units.fuelType);     end;
    if (Units.currencyType > -1) then setXMLInt(xmlFile, rootTag..".config#currencyType",  Units.currencyType); end;
    saveXMLFile(xmlFile);   -- TODO - How to determine if the file was saved or not?
    delete(xmlFile);
end;

-- Re-implementation of Steerable.draw(), to alter the speed & fuel units and calculation.
function Units.steerableDraw(self)
    local kmh = math.min(100, math.max(0, self.lastSpeed*self.speedDisplayScale*3600));
    local maxSpeed = 80;

    self.hudBackgroundOverlay:render();
    setTextBold(true);
    setTextColor(1.0, 1.0, 1.0, 1.0);
    setTextAlignment(RenderText.ALIGN_CENTER);

    local speedInDisplayUnits = kmh / Units.speedFactor;

    setTextColor(0, 0, 0, 1);
    renderText(self.hudBasePosX + self.hudBaseWidth / 2 + 0.002, self.hudBasePosY + 0.005 + 3 * 0.0396, 0.024, string.format("%2d " .. Units.speedUnit, speedInDisplayUnits));
    setTextColor(1, 1, 1, 1);
    renderText(self.hudBasePosX + self.hudBaseWidth / 2 + 0.002, self.hudBasePosY + 0.008 + 3 * 0.0396, 0.024, string.format("%2d " .. Units.speedUnit, speedInDisplayUnits));

    self.hudBarRedOverlay.width = self.hudBarWidth * math.min(1, kmh / maxSpeed);
    setOverlayUVs(self.hudBarRedOverlay.overlayId, 0, 0.05, 0, 1, math.min(1, kmh / maxSpeed), 0.05, math.min(1, kmh / maxSpeed), 1);
    self.hudBarRedOverlay:render();

    setTextColor(0, 0, 0, 1);
    renderText(self.hudBasePosX + self.hudBaseWidth / 2 + 0.002, self.hudBasePosY + 0.005 + 2 * 0.0396, 0.024, string.format("%1.0f " .. g_i18n:getText("Currency_symbol"), g_currentMission.missionStats.money));
    setTextColor(1, 1, 1, 1);
    renderText(self.hudBasePosX + self.hudBaseWidth / 2 + 0.002, self.hudBasePosY + 0.008 + 2 * 0.0396, 0.024, string.format("%1.0f " .. g_i18n:getText("Currency_symbol"), g_currentMission.missionStats.money));

    local currentFuelPercentage = 0;
    local fuelWarnPercentage = 20;
    if self.fuelCapacity > 0 then
        currentFuelPercentage = (self.fuelFillLevel / self.fuelCapacity + 0.0001) * 100;
    end;

    local fuelInDisplayUnits = self.fuelFillLevel / Units.fuelFactor;

    setTextColor(0, 0, 0, 1);
    renderText(self.hudBasePosX + self.hudBaseWidth / 2 + 0.002, self.hudBasePosY + 0.005, 0.024, string.format("%d " .. Units.fuelUnit, fuelInDisplayUnits));
    if currentFuelPercentage < fuelWarnPercentage then
        setTextColor(1, 0, 0, 1);
    else
        setTextColor(1, 1, 1, 1);
    end;
    renderText(self.hudBasePosX + self.hudBaseWidth / 2 + 0.002, self.hudBasePosY + 0.008, 0.024, string.format("%d " .. Units.fuelUnit, fuelInDisplayUnits));

    if currentFuelPercentage < fuelWarnPercentage then
        setTextColor(1, 1, 1, 1);
    end;

    self.hudBarGreenOverlay.width = self.hudBarWidth * (self.fuelFillLevel / self.fuelCapacity); -- TODO: check for division by zero!
    setOverlayUVs(self.hudBarGreenOverlay.overlayId, 0, 0.05, 0, 1, self.fuelFillLevel / self.fuelCapacity, 0.05, self.fuelFillLevel / self.fuelCapacity, 1);
    self.hudBarGreenOverlay:render();

    local trailerFillLevel, trailerCapacity = self:getAttachedTrailersFillLevelAndCapacity();
    if trailerFillLevel ~= nil and trailerCapacity ~= nil and trailerCapacity > 0 then
        self:drawGrainLevel(trailerFillLevel, trailerCapacity, 101);
    else
        -- if we don't have a trailer attached and we're not in a combine, render the frames overlay now
        if self.grainTankFillLevel == nil then
            self.hudFramesOverlay:render();
        end;
    end;

    setTextAlignment(RenderText.ALIGN_LEFT);
    setTextBold(false);

    if self.showWaterWarning then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_depth_into_the_water"), 0.05, 0.025+0.007);
    end;

    if table.getn(self.attachedImplements) > 1 then
        g_currentMission:addHelpButtonText(g_i18n:getText("Change_tools"), InputBinding.SWITCH_IMPLEMENT);
    end;
end;

-- Overwrite the draw() function of Steerable.
Steerable.draw = Units.steerableDraw;

-- Append the Units.save() function, to when saving game.
CareerScreen.saveSelectedGame = Utils.appendedFunction(CareerScreen.saveSelectedGame, Units.save);

--
addModEventListener(Units);
